#ifndef MANDELBROT_H
#define MANDELBROT_H

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include "mandelbrot-gui.h"     /* has setup(), interact() */

/* Default values for things. */
#define N           2           /* size of problem space (x, y from -N to N) */
#define NPIXELS     800         /* size of display window in pixels */


/* Structure definition for complex numbers */
typedef struct {
  double r, i;
} complex ;

/* Shorthand for some commonly-used types */
typedef unsigned int uint;
typedef unsigned long ulong;

/* Boundaries of the Mandelbrot set to display */
extern double r_min;
extern double r_max;
extern double i_min;
extern double i_max;
extern uint width;         /* dimensions of display window */
extern uint height;
extern double scale_r, scale_i, scale_color;
extern uint maxiter;
extern ulong min_color, max_color;

extern Display *display;
extern GC gc;
extern Window win;

#define TIME_DIFF(t1, t2) (((t2).tv_sec-(t1).tv_sec)*1e6+((t2).tv_usec-(t1).tv_usec))

/* Compute the color of point (row, col) */
static ulong compute(int row, int col) {
  complex z;
  complex c;
  ulong couleur;
  uint k;
  double lengthsq, temp;

  z.r = z.i = 0;
  /* Scale display coordinates to actual region  */
  c.r = r_min + ((double) col * scale_r);
  c.i = i_min + ((double) (height-1-row) * scale_i);
  /* height-1-row so y axis displays
   * with larger values at top
   */

  /* Calculate z0, z1, .... until divergence or maximum iterations */
  k = 0;
  do  {
    temp = z.r*z.r - z.i*z.i + c.r;
    z.i = 2*z.r*z.i + c.i;
    z.r = temp;
    lengthsq = z.r*z.r + z.i*z.i;
    ++k;
  } while (lengthsq < (N*N) && k < maxiter);

  /* Scale color and display point  */
  couleur = (ulong) ((k-1) * scale_color) + min_color;
  return couleur;
}

void init_mandelbrot() {
  /* Initialize for graphical display */
  int setup_return = setup(width, height, &display, &win, &gc,
			   &min_color, &max_color);
  if (setup_return != EXIT_SUCCESS) {
    fprintf(stderr, "Unable to initialize display, continuing\n");
    exit(EXIT_FAILURE);
  }

  /* Compute factors to scale computational region to window */
  scale_r = (double) (r_max - r_min) / (double) width;
  scale_i = (double) (i_max - i_min) / (double) height;

  /* Compute factor for color scaling */
  scale_color = (double) (max_color - min_color) / (double) (maxiter - 1);

}
#endif	/* MANDELBROT_H */

